@@ -1,8 +1,7 @@
-// Copyright 2018 Frédéric Guillot. All rights reserved.
-// Use of this source code is governed by the Apache 2.0
-// license that can be found in the LICENSE file.
+// SPDX-FileCopyrightText: Copyright The Miniflux Authors. All rights reserved.
+// SPDX-License-Identifier: Apache-2.0
 
-package client // import "miniflux.app/client"
+package client // import "miniflux.app/v2/client"
 
 import (
 	"encoding/json"
@@ -10,6 +9,7 @@ import (
 	"io"
 	"net/url"
 	"strconv"
+	"strings"
 )
 
 // Client holds API procedure calls.
@@ -18,11 +18,60 @@ type Client struct {
 }
 
 // New returns a new Miniflux client.
+// Deprecated: use NewClient instead.
 func New(endpoint string, credentials ...string) *Client {
-	if len(credentials) == 2 {
+	return NewClient(endpoint, credentials...)
+}
+
+// NewClient returns a new Miniflux client.
+func NewClient(endpoint string, credentials ...string) *Client {
+	// Trim trailing slashes and /v1 from the endpoint.
+	endpoint = strings.TrimSuffix(endpoint, "/")
+	endpoint = strings.TrimSuffix(endpoint, "/v1")
+	switch len(credentials) {
+	case 2:
 		return &Client{request: &request{endpoint: endpoint, username: credentials[0], password: credentials[1]}}
+	case 1:
+		return &Client{request: &request{endpoint: endpoint, apiKey: credentials[0]}}
+	default:
+		return &Client{request: &request{endpoint: endpoint}}
 	}
-	return &Client{request: &request{endpoint: endpoint, apiKey: credentials[0]}}
+}
+
+// Healthcheck checks if the application is up and running.
+func (c *Client) Healthcheck() error {
+	body, err := c.request.Get("/healthcheck")
+	if err != nil {
+		return fmt.Errorf("miniflux: unable to perform healthcheck: %w", err)
+	}
+	defer body.Close()
+
+	responseBodyContent, err := io.ReadAll(body)
+	if err != nil {
+		return fmt.Errorf("miniflux: unable to read healthcheck response: %w", err)
+	}
+
+	if string(responseBodyContent) != "OK" {
+		return fmt.Errorf("miniflux: invalid healthcheck response: %q", responseBodyContent)
+	}
+
+	return nil
+}
+
+// Version returns the version of the Miniflux instance.
+func (c *Client) Version() (*VersionResponse, error) {
+	body, err := c.request.Get("/v1/version")
+	if err != nil {
+		return nil, err
+	}
+	defer body.Close()
+
+	var versionResponse *VersionResponse
+	if err := json.NewDecoder(body).Decode(&versionResponse); err != nil {
+		return nil, fmt.Errorf("miniflux: json error (%v)", err)
+	}
+
+	return versionResponse, nil
 }
 
 // Me returns the logged user information.
@@ -34,8 +83,7 @@ func (c *Client) Me() (*User, error) {
 	defer body.Close()
 
 	var user *User
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&user); err != nil {
+	if err := json.NewDecoder(body).Decode(&user); err != nil {
 		return nil, fmt.Errorf("miniflux: json error (%v)", err)
 	}
 
@@ -51,8 +99,7 @@ func (c *Client) Users() (Users, error) {
 	defer body.Close()
 
 	var users Users
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&users); err != nil {
+	if err := json.NewDecoder(body).Decode(&users); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -68,8 +115,7 @@ func (c *Client) UserByID(userID int64) (*User, error) {
 	defer body.Close()
 
 	var user User
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&user); err != nil {
+	if err := json.NewDecoder(body).Decode(&user); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -85,8 +131,7 @@ func (c *Client) UserByUsername(username string) (*User, error) {
 	defer body.Close()
 
 	var user User
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&user); err != nil {
+	if err := json.NewDecoder(body).Decode(&user); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -106,8 +151,7 @@ func (c *Client) CreateUser(username, password string, isAdmin bool) (*User, err
 	defer body.Close()
 
 	var user *User
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&user); err != nil {
+	if err := json.NewDecoder(body).Decode(&user); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -123,8 +167,7 @@ func (c *Client) UpdateUser(userID int64, userChanges *UserModificationRequest)
 	defer body.Close()
 
 	var u *User
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&u); err != nil {
+	if err := json.NewDecoder(body).Decode(&u); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -151,8 +194,7 @@ func (c *Client) Discover(url string) (Subscriptions, error) {
 	defer body.Close()
 
 	var subscriptions Subscriptions
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&subscriptions); err != nil {
+	if err := json.NewDecoder(body).Decode(&subscriptions); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -168,8 +210,7 @@ func (c *Client) Categories() (Categories, error) {
 	defer body.Close()
 
 	var categories Categories
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&categories); err != nil {
+	if err := json.NewDecoder(body).Decode(&categories); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -181,15 +222,13 @@ func (c *Client) CreateCategory(title string) (*Category, error) {
 	body, err := c.request.Post("/v1/categories", map[string]interface{}{
 		"title": title,
 	})
-
 	if err != nil {
 		return nil, err
 	}
 	defer body.Close()
 
 	var category *Category
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&category); err != nil {
+	if err := json.NewDecoder(body).Decode(&category); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -201,15 +240,13 @@ func (c *Client) UpdateCategory(categoryID int64, title string) (*Category, erro
 	body, err := c.request.Put(fmt.Sprintf("/v1/categories/%d", categoryID), map[string]interface{}{
 		"title": title,
 	})
-
 	if err != nil {
 		return nil, err
 	}
 	defer body.Close()
 
 	var category *Category
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&category); err != nil {
+	if err := json.NewDecoder(body).Decode(&category); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -231,8 +268,7 @@ func (c *Client) CategoryFeeds(categoryID int64) (Feeds, error) {
 	defer body.Close()
 
 	var feeds Feeds
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&feeds); err != nil {
+	if err := json.NewDecoder(body).Decode(&feeds); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -244,6 +280,12 @@ func (c *Client) DeleteCategory(categoryID int64) error {
 	return c.request.Delete(fmt.Sprintf("/v1/categories/%d", categoryID))
 }
 
+// RefreshCategory refreshes a category.
+func (c *Client) RefreshCategory(categoryID int64) error {
+	_, err := c.request.Put(fmt.Sprintf("/v1/categories/%d/refresh", categoryID), nil)
+	return err
+}
+
 // Feeds gets all feeds.
 func (c *Client) Feeds() (Feeds, error) {
 	body, err := c.request.Get("/v1/feeds")
@@ -253,8 +295,7 @@ func (c *Client) Feeds() (Feeds, error) {
 	defer body.Close()
 
 	var feeds Feeds
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&feeds); err != nil {
+	if err := json.NewDecoder(body).Decode(&feeds); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -292,8 +333,7 @@ func (c *Client) Feed(feedID int64) (*Feed, error) {
 	defer body.Close()
 
 	var feed *Feed
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&feed); err != nil {
+	if err := json.NewDecoder(body).Decode(&feed); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -313,8 +353,7 @@ func (c *Client) CreateFeed(feedCreationRequest *FeedCreationRequest) (int64, er
 	}
 
 	var r result
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&r); err != nil {
+	if err := json.NewDecoder(body).Decode(&r); err != nil {
 		return 0, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -369,8 +408,7 @@ func (c *Client) FeedIcon(feedID int64) (*FeedIcon, error) {
 	defer body.Close()
 
 	var feedIcon *FeedIcon
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&feedIcon); err != nil {
+	if err := json.NewDecoder(body).Decode(&feedIcon); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -386,8 +424,7 @@ func (c *Client) FeedEntry(feedID, entryID int64) (*Entry, error) {
 	defer body.Close()
 
 	var entry *Entry
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&entry); err != nil {
+	if err := json.NewDecoder(body).Decode(&entry); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -403,8 +440,7 @@ func (c *Client) CategoryEntry(categoryID, entryID int64) (*Entry, error) {
 	defer body.Close()
 
 	var entry *Entry
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&entry); err != nil {
+	if err := json.NewDecoder(body).Decode(&entry); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -420,8 +456,7 @@ func (c *Client) Entry(entryID int64) (*Entry, error) {
 	defer body.Close()
 
 	var entry *Entry
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&entry); err != nil {
+	if err := json.NewDecoder(body).Decode(&entry); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -439,8 +474,7 @@ func (c *Client) Entries(filter *Filter) (*EntryResultSet, error) {
 	defer body.Close()
 
 	var result EntryResultSet
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&result); err != nil {
+	if err := json.NewDecoder(body).Decode(&result); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -458,8 +492,7 @@ func (c *Client) FeedEntries(feedID int64, filter *Filter) (*EntryResultSet, err
 	defer body.Close()
 
 	var result EntryResultSet
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&result); err != nil {
+	if err := json.NewDecoder(body).Decode(&result); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -477,8 +510,7 @@ func (c *Client) CategoryEntries(categoryID int64, filter *Filter) (*EntryResult
 	defer body.Close()
 
 	var result EntryResultSet
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&result); err != nil {
+	if err := json.NewDecoder(body).Decode(&result); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
@@ -496,13 +528,54 @@ func (c *Client) UpdateEntries(entryIDs []int64, status string) error {
 	return err
 }
 
+// UpdateEntry updates an entry.
+func (c *Client) UpdateEntry(entryID int64, entryChanges *EntryModificationRequest) (*Entry, error) {
+	body, err := c.request.Put(fmt.Sprintf("/v1/entries/%d", entryID), entryChanges)
+	if err != nil {
+		return nil, err
+	}
+	defer body.Close()
+
+	var entry *Entry
+	if err := json.NewDecoder(body).Decode(&entry); err != nil {
+		return nil, fmt.Errorf("miniflux: response error (%v)", err)
+	}
+
+	return entry, nil
+}
+
 // ToggleBookmark toggles entry bookmark value.
 func (c *Client) ToggleBookmark(entryID int64) error {
 	_, err := c.request.Put(fmt.Sprintf("/v1/entries/%d/bookmark", entryID), nil)
 	return err
 }
 
-// FetchCounters
+// SaveEntry sends an entry to a third-party service.
+func (c *Client) SaveEntry(entryID int64) error {
+	_, err := c.request.Post(fmt.Sprintf("/v1/entries/%d/save", entryID), nil)
+	return err
+}
+
+// FetchEntryOriginalContent fetches the original content of an entry using the scraper.
+func (c *Client) FetchEntryOriginalContent(entryID int64) (string, error) {
+	body, err := c.request.Get(fmt.Sprintf("/v1/entries/%d/fetch-content", entryID))
+	if err != nil {
+		return "", err
+	}
+	defer body.Close()
+
+	var response struct {
+		Content string `json:"content"`
+	}
+
+	if err := json.NewDecoder(body).Decode(&response); err != nil {
+		return "", fmt.Errorf("miniflux: response error (%v)", err)
+	}
+
+	return response.Content, nil
+}
+
+// FetchCounters fetches feed counters.
 func (c *Client) FetchCounters() (*FeedCounters, error) {
 	body, err := c.request.Get("/v1/feeds/counters")
 	if err != nil {
@@ -511,14 +584,35 @@ func (c *Client) FetchCounters() (*FeedCounters, error) {
 	defer body.Close()
 
 	var result FeedCounters
-	decoder := json.NewDecoder(body)
-	if err := decoder.Decode(&result); err != nil {
+	if err := json.NewDecoder(body).Decode(&result); err != nil {
 		return nil, fmt.Errorf("miniflux: response error (%v)", err)
 	}
 
 	return &result, nil
 }
 
+// FlushHistory changes all entries with the status "read" to "removed".
+func (c *Client) FlushHistory() error {
+	_, err := c.request.Put("/v1/flush-history", nil)
+	return err
+}
+
+// Icon fetches a feed icon.
+func (c *Client) Icon(iconID int64) (*FeedIcon, error) {
+	body, err := c.request.Get(fmt.Sprintf("/v1/icons/%d", iconID))
+	if err != nil {
+		return nil, err
+	}
+	defer body.Close()
+
+	var feedIcon *FeedIcon
+	if err := json.NewDecoder(body).Decode(&feedIcon); err != nil {
+		return nil, fmt.Errorf("miniflux: response error (%v)", err)
+	}
+
+	return feedIcon, nil
+}
+
 func buildFilterQueryString(path string, filter *Filter) string {
 	if filter != nil {
 		values := url.Values{}
@@ -547,14 +641,30 @@ func buildFilterQueryString(path string, filter *Filter) string {
 			values.Set("after", strconv.FormatInt(filter.After, 10))
 		}
 
-		if filter.AfterEntryID > 0 {
-			values.Set("after_entry_id", strconv.FormatInt(filter.AfterEntryID, 10))
-		}
-
 		if filter.Before > 0 {
 			values.Set("before", strconv.FormatInt(filter.Before, 10))
 		}
 
+		if filter.PublishedAfter > 0 {
+			values.Set("published_after", strconv.FormatInt(filter.PublishedAfter, 10))
+		}
+
+		if filter.PublishedBefore > 0 {
+			values.Set("published_before", strconv.FormatInt(filter.PublishedBefore, 10))
+		}
+
+		if filter.ChangedAfter > 0 {
+			values.Set("changed_after", strconv.FormatInt(filter.ChangedAfter, 10))
+		}
+
+		if filter.ChangedBefore > 0 {
+			values.Set("changed_before", strconv.FormatInt(filter.ChangedBefore, 10))
+		}
+
+		if filter.AfterEntryID > 0 {
+			values.Set("after_entry_id", strconv.FormatInt(filter.AfterEntryID, 10))
+		}
+
 		if filter.BeforeEntryID > 0 {
 			values.Set("before_entry_id", strconv.FormatInt(filter.BeforeEntryID, 10))
 		}
