@@ -193,7 +193,14 @@ func (s *Storage) Integration(userID int64) (*model.Integration, error) {
 			rssbridge_url,
 			omnivore_enabled,
 			omnivore_api_key,
-			omnivore_url
+			omnivore_url,
+			raindrop_enabled,
+			raindrop_token,
+			raindrop_collection_id,
+			raindrop_tags,
+			betula_enabled,
+			betula_url,
+			betula_token
 		FROM
 			integrations
 		WHERE
@@ -286,6 +293,13 @@ func (s *Storage) Integration(userID int64) (*model.Integration, error) {
 		&integration.OmnivoreEnabled,
 		&integration.OmnivoreAPIKey,
 		&integration.OmnivoreURL,
+		&integration.RaindropEnabled,
+		&integration.RaindropToken,
+		&integration.RaindropCollectionID,
+		&integration.RaindropTags,
+		&integration.BetulaEnabled,
+		&integration.BetulaURL,
+		&integration.BetulaToken,
 	)
 	switch {
 	case err == sql.ErrNoRows:
@@ -386,9 +400,16 @@ func (s *Storage) UpdateIntegration(integration *model.Integration) error {
 			omnivore_url=$81,
 			linkwarden_enabled=$82,
 			linkwarden_url=$83,
-			linkwarden_api_key=$84
+			linkwarden_api_key=$84,
+			raindrop_enabled=$85,
+			raindrop_token=$86,
+			raindrop_collection_id=$87,
+			raindrop_tags=$88,
+			betula_enabled=$89,
+			betula_url=$90,
+			betula_token=$91
 		WHERE
-			user_id=$85
+			user_id=$92
 	`
 	_, err := s.db.Exec(
 		query,
@@ -476,6 +497,13 @@ func (s *Storage) UpdateIntegration(integration *model.Integration) error {
 		integration.LinkwardenEnabled,
 		integration.LinkwardenURL,
 		integration.LinkwardenAPIKey,
+		integration.RaindropEnabled,
+		integration.RaindropToken,
+		integration.RaindropCollectionID,
+		integration.RaindropTags,
+		integration.BetulaEnabled,
+		integration.BetulaURL,
+		integration.BetulaToken,
 		integration.UserID,
 	)
 
@@ -513,7 +541,9 @@ func (s *Storage) HasSaveEntry(userID int64) (result bool) {
 				readeck_enabled='t' OR
 				shaarli_enabled='t' OR
 				webhook_enabled='t' OR
-				omnivore_enabled='t'
+				omnivore_enabled='t' OR
+				raindrop_enabled='t' OR
+				betula_enabled='t'
 			)
 	`
 	if err := s.db.QueryRow(query, userID).Scan(&result); err != nil {
